import {createEnhancedElement} from '@servicenow/library-enhanced-element';
import styles from './_now-highlighted-value.scss';
import '@servicenow/now-icon';

/*
 * TODO: This is a temporary way of debugging customer problems during the
 * Seismic 18 upgrade process, because tectonic resolution makes it hard to
 * know what version of a component is running. Remove when not needed.
 * Intentionally runs in dev + production environments so we can debug all builds.
 */
const {name, version} = require('../package.json');
const registry = (window.___NOW_DESIGN_SYSTEM_PACKAGES___ =
	window.___NOW_DESIGN_SYSTEM_PACKAGES___ || {});
if (!registry[name]) {
	registry[name] = version;
}

const renderIcon = (icon, showIcon, dir) => {
	if (icon || showIcon) {
		return (
			<now-icon
				icon={icon ? icon : 'presence-fill'}
				dir={dir}
				size="sm"
				className="now-highlighted-value-icon"
				aria-hidden="true"
			/>
		);
	}
};

const view = (state) => {
	const {
		label = '',
		status,
		color,
		variant,
		icon,
		showIcon,
		dir
	} = state.properties;
	const statusOrColor = status || color;
	return (
		<span
			class={{
				'now-highlighted-value': true,
				['-' + statusOrColor]: statusOrColor,
				['-' + variant]: variant
			}}
			title={label}>
			{renderIcon(icon, showIcon, dir)}
			<span className="now-line-height-crop">
				<span className="now-highlighted-value-label">{label}</span>
			</span>
		</span>
	);
};

/**
 * Highlighted values communicate the value of an item
 * by promoting visibility and readability.
 *
 * Use highlighted values when you want to convey the status in a certain area,
 * such as a card, list, or widget. A highlighted value may be combined with a status icon.
 *
 * ```jsx
 * <now-highlighted-value label="Value" color="warning" show-icon>
 * </now-highlighted-value>
 *
 * <now-highlighted-value label="Tag" color="blue" variant="secondary" icon="clock-outline">
 * </now-highlighted-value>
 * ```
 *
 * @seismicElement now-highlighted-value
 * @summary A styled label with an optional icon.
 */
createEnhancedElement('now-highlighted-value', {
	properties: {
		/**
		 * Text displayed as the highlighted value.
		 * @type {string}
		 */
		label: {required: true, schema: {type: 'string'}},
		/**
		 * Deprecated. Use `color` instead.
		 * @private
		 */
		status: {},
		/**
		 * Sets the highlighted value background color.
		 * @type {('critical'|'high'|'warning'|'moderate'|'positive'|'info'|'low'|'magenta'|'pink'|'orange'|'yellow'|'brown'|'green'|'green-yellow'|'blue'|'gray'|'teal'|'purple')}
		 */
		color: {
			default: 'low',
			schema: {
				type: 'string',
				enum: [
					'critical',
					'high',
					'warning',
					'moderate',
					'positive',
					'info',
					'low',
					'magenta',
					'pink',
					'orange',
					'yellow',
					'brown',
					'green',
					'green-yellow',
					'blue',
					'gray',
					'teal',
					'purple'
				]
			}
		},
		/**
		 * Sets the shade of the highlighted value background and the color of
		 * the text and/or icon.
		 * @type {('primary'|'secondary'|'tertiary')}
		 */
		variant: {
			default: 'tertiary',
			schema: {type: 'string', enum: ['primary', 'secondary', 'tertiary']}
		},
		/**
		 * Flag that determines whether an icon displays with the label.
		 * @type {boolean}
		 */
		showIcon: {default: false, schema: {type: 'boolean'}},
		/**
		 * When `show-icon` is set, a solid circle will be displayed in the
		 * highlighted value. Set this property with a valid icon name from
		 * the `now-icon` set to display a custom icon instead.
		 * @type {string}
		 */
		icon: {schema: {type: 'string'}}
	},
	view,
	styles
});
